from ngsolve import *
from netgen.csg import *
from ngsolve.internal import visoptions

left  = Plane (Pnt(0,0,0), Vec(-1,0,0) ).bc("left")
right = Plane (Pnt(1,1,1), Vec( 1,0,0) ).bc("right")
front = Plane (Pnt(0,0,0), Vec(0,-1,0) ).bc("front")
back  = Plane (Pnt(1,1,1), Vec(0, 1,0) ).bc("back")
bot   = Plane (Pnt(0,0,0), Vec(0,0,-1) ).bc("bot")
top   = Plane (Pnt(1,1,1), Vec(0,0, 1) ).bc("top")
cube = left * right * front * back * bot * top
geo = CSGeometry()
geo.Add (cube)
mesh = Mesh(geo.GenerateMesh(maxh=0.3))
Draw(mesh)


fes1 = HCurl(mesh, order=1, dirichlet="left|right|top|bottom|front|back")
fes2 = HCurl(mesh, order=1, dirichlet="left|right|top|bottom|front|back")
fes = FESpace( [fes1,fes2] )
(A,B), (dA,dB) = fes.TnT()

gfsol    = GridFunction(fes)
gfsolold = GridFunction(fes)

gfA, gfB  = gfsol.components
Aold,Bold = gfsolold.components

tau = Parameter(0.001)

c = 1
e = 5
d = 3
f = 4

a = BilinearForm(fes)
a += SymbolicBFI( c/tau*(A-Aold)*dB + e/tau*(B-Bold)*dB - B*curl(dB) )
a += SymbolicBFI( d/tau*(A-Aold)*dA + f/tau*(B-Bold)*dA + A*curl(dA) )

Q1 = CoefficientFunction( (1,1,1) )
Q2 = CoefficientFunction( (0,10,0) )
a += SymbolicBFI( -Q1*dA - Q2*dB )

#Set initial data
#gfA.Set(...)
#gfB.Set(...)

tend = 1
t = 0

Draw(gfA, mesh, "A")
Draw(gfB, mesh, "B")

visoptions.scalfunction='A:0'

while t < tend -tau.Get()/2:
    print("t = ", t)
    t += tau.Get()

    gfsolold.vec.data = gfsol.vec

    with TaskManager():
        solvers.Newton(a, gfsol, maxerr=1e-10, maxit=3, printing=True)

    Redraw()
